' Create route indexes for RW_Tools
' (c) 2013 by Thomas Klaeger
' V2.0.0 - include markers spanning multiple track ribbons

Option Explicit

Dim RWFolder, ToolsFolder, fso, WshShell, WorkFolder

Init
ReadConfig
RisePrivilege

Dim startTime, endTime, routeCount
startTime = Time : routeCount = 0
Dim routeFolder, routeFolders, route
set routeFolder = fso.getFolder(RWFolder & "Content\Routes")
set routeFolders = routeFolder.SubFolders
For Each route in routeFolders
  'WScript.Echo route.name
  IndexRoute route.path
Next

fso.DeleteFile(WorkFolder & "tracks.xml")

endTime = Time

WScript.Echo "Indexed "&routeCount&" routes in "&FormatNumber((endTime-startTime)*86400,1)&" seconds"


Public Sub IndexRoute(ByRef route)
  Dim xmlDoc, routeName, lines, refIds
  Set xmlDoc = CreateObject("Msxml2.DOMDocument.6.0")
  xmlDoc.async = false
  If Not xmlDoc.load(route & "\RouteProperties.xml") Then
    Exit Sub
  End If
  routeName = xmlDoc.selectSingleNode("//English").text
  routeCount = routeCount + 1

  ' Convert Tracks.bin to xml
  WshShell.Run RWFolder & "serz.exe """ & route & "\Networks\Tracks.bin"" /xml:""" & WorkFolder & "tracks.xml""", 0, true

  ' Read the Tracks.xml
  xmlDoc.async = false
  xmlDoc.load(WorkFolder & "Tracks.xml")

  Set lines = CreateObject("Scripting.Dictionary")
  Set refIds = CreateObject("Scripting.Dictionary")

  ' Extract Markers
  ExtractIdAndMarkers lines, refIds, xmlDoc.selectNodes("//Network-cTrackRibbon")

  WriteRouteIndex routeName, lines
End Sub


Public Sub ExtractIdAndMarkers(ByRef lines, ByRef refIds, ByRef Ribbons)
  Dim ribbon
  For Each ribbon In ribbons
    Dim ribbonId
    ribbonId = ribbon.selectSingleNode("RibbonID/cGUID/DevString").text
    ExtractMarkers lines, refIds, ribbonId, ribbon
  Next
End Sub


Public Sub ExtractMarkers(ByRef lines, ByRef refIds, ByRef ribbonId, ByRef ribbon) 
  Dim properties, property, nameNode, markers, ownedEntity, namePath
  Set properties = ribbon.selectNodes("Properties/Network-cPropertyContainer/Property/*")
  markers = ""
  For Each property in properties
    namePath = ""
    If property.nodeName = "Network-cTrackMarkerProperty" Then
      namePath = "Component/Network-cTrackMarkerComponent/DisplayName"
      Set ownedEntity = property.selectSingleNode("Entity/cOwnedEntity")
    ElseIf property.nodeName = "Network-cPortalProperty" Then
      namePath = "Component/Network-cTrackMarkerComponent/DisplayName"
      Set ownedEntity = property.selectSingleNode("Entity/cOwnedEntity")
    ElseIf property.nodeName = "Network-cTransferPointProperty" Then
      namePath = "Component/Network-cTransferPointComponent/DisplayName"
      Set ownedEntity = property.selectSingleNode("Entity/cOwnedEntity")
    End If
    If Not namePath = "" Then
      If ownedEntity is Nothing Then
        'WScript.Echo "On Ribbon " & ribbonId & " found " & property.nodeName & " without proper cOwnedEntity"
	markers = markers & " - " & "Unnamed " & Mid(property.nodeName, 10, len(property.nodeName)-17)
      ElseIf ownedEntity.getAttribute("d:type") = "ref" Then
        markers = markers & " - " & refIds.Item(ownedEntity.text)
      Else
        Set nameNode = ownedEntity.selectSingleNode(namePath)
	refIds.Add ownedEntity.getAttribute("d:id"), nameNode.text
	markers = markers & " - " & nameNode.text
      End If
    End If
  Next
  If Len(markers) > 0 Then
    lines.Add lines.count, """" & ribbonId & """,""" & Mid(markers, 4) & """"
  End If
End Sub


Public Sub WriteRouteIndex(ByRef routeName, ByRef lines)
  Dim line, routeIndex
  Set routeIndex = fso.CreateTextFile("Index/" & routeName & ".txt", true, false)
  routeIndex.Write " "
  routeIndex.WriteLine lines.Count
  For Each line in lines.Items
    routeIndex.WriteLine line
  Next
  routeIndex.Close
End Sub


Public Sub Init
  Set fso = CreateObject("Scripting.FileSystemObject")
  Set WshShell = CreateObject("WScript.Shell")
  Dim scriptPath, scriptName
  scriptName = WScript.ScriptFullName
  scriptPath = fso.getParentFolderName(scriptName)
  ToolsFOlder = scriptPath
  WorkFolder = ToolsFolder & "\TempFiles\"
  WShShell.CurrentDirectory = scriptPath
End Sub

Public Sub ReadConfig
  Dim dict, file, row, line
  Set dict = CreateObject("Scripting.Dictionary")
  Set file = fso.OpenTextFile ("RWpreferences.txt", 1)
  row = 0
  Do Until file.AtEndOfStream
    line = file.Readline
    dict.Add row, line
    row = row + 1
  Loop
  file.Close

  RWFolder = dict.Item(6)
  RWFolder = Mid(RWFolder, 2, len(RWFolder)-2) & "\"
End Sub

public Sub RisePrivilege
  On Error Resume Next
  Dim key, objShell
  key = CreateObject("WScript.Shell").RegRead("HKEY_USERS\s-1-5-19\")
  If err.number <> 0 Then
    Set objShell = CreateObject("Shell.Application")
    objShell.ShellExecute "wscript.exe", Chr(34) & WScript.ScriptFullName & Chr(34), "", "runas", 1
    WScript.Quit 0
  End If
  On Error Goto 0
End Sub